
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: HIGHLIGHT_FONT_COLOR, Mixin, ToggleCharacter

local BreakUpLargeNumbers = BreakUpLargeNumbers
local C_CurrencyInfo = C_CurrencyInfo
local floor = floor
local MAX_PLAYER_LEVEL = MAX_PLAYER_LEVEL

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display Valor")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI Valor", {
	description = "Shows while you're below the Valor cap.",
	type = "progress bar",
	barR = C.VALOR_RESOURCE_COLOR.r, barG = C.VALOR_RESOURCE_COLOR.g, barB = C.VALOR_RESOURCE_COLOR.b,
	label = "Valor",
	level = UNKNOWN,
	text = UNKNOWN,
	value = "0",
	suffix = "%",
	visible = false,
})

local INITIAL_CAP = 1500
local RECURRING_CAP = 750
local WEEK_TEXT = "Week %d"

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function GetWeeklyContext(currencyInfo)
	return WEEK_TEXT:format(currencyInfo.maxQuantity == INITIAL_CAP and 1 or currencyInfo.maxQuantity / RECURRING_CAP - 1)
end

local function GetWeeklyValues(currencyInfo)
	local maxQuantity = currencyInfo.maxQuantity == INITIAL_CAP and INITIAL_CAP or RECURRING_CAP
	local minQuantity = currencyInfo.maxQuantity - maxQuantity
	local totalEarned = currencyInfo.totalEarned - minQuantity
	local displayWeek = currencyInfo.totalEarned >= minQuantity

	if displayWeek then
		return totalEarned, maxQuantity, GetWeeklyContext(currencyInfo)
	else
		return currencyInfo.totalEarned, currencyInfo.maxQuantity
	end
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	if not C.IS_RETAIL then
		self:Disable()
		return
	end

	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", "Update")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE", "OnConfigUpdate")
	self:Update()
end

function M:OnConfigUpdate(event, key, ...)
	if key == "color.resource.valor" then
		Mixin(O, {barR = C.VALOR_RESOURCE_COLOR.r, barG = C.VALOR_RESOURCE_COLOR.g, barB = C.VALOR_RESOURCE_COLOR.b})
	end
end

function M:ShouldBeVisible()
	local currencyInfo = C_CurrencyInfo.GetCurrencyInfo(1191)
	return currencyInfo
		and currencyInfo.discovered
		and currencyInfo.maxQuantity > 0
		and currencyInfo.maxQuantity > currencyInfo.totalEarned
		and C.PLAYER_LEVEL >= MAX_PLAYER_LEVEL
end

function M:Update(event)
	local currencyInfo = C_CurrencyInfo.GetCurrencyInfo(1191)

	if not currencyInfo then
		O.visible = false
		return
	end

	local value, max, level = GetWeeklyValues(currencyInfo)
	local percent = (value / max) * 100

	O.level = level
	O.text = C.XP_TEXT:format(BreakUpLargeNumbers(value), BreakUpLargeNumbers(max), floor(percent))
	O.value = percent
	O.visible = M:ShouldBeVisible()
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnClick(frame, button)
	ToggleCharacter("TokenFrame")
end

function O.OnTooltipShow(tooltip)
	tooltip:SetText(O.label)
	tooltip:AddLine(O.text, HIGHLIGHT_FONT_COLOR:GetRGB())
	tooltip:AddLine(O.level)
end
